#!/usr/bin/env ruby

#-----------------------------------------------------------------------------#
#   flickr.rb                                                                 #
#                                                                             #
#   Copyright (c) 2012, Rajiv Bakulesh Shah, original author.                 #
#                                                                             #
#       This file is free software; you can redistribute it and/or modify     #
#       it under the terms of the GNU General Public License as published     #
#       by the Free Software Foundation, either version 3 of the License,     #
#       or (at your option) any later version.                                #
#                                                                             #
#       This file is distributed in the hope that it will be useful, but      #
#       WITHOUT ANY WARRANTY; without even the implied warranty of            #
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     #
#       General Public License for more details.                              #
#                                                                             #
#       You should have received a copy of the GNU General Public License     #
#       along with this file.  If not, see:                                   #
#           <http://www.gnu.org/licenses/>.                                   #
#-----------------------------------------------------------------------------#


require 'flickraw-cached'
require 'logger'

load 'timer.rb'


module Flickr
  API_KEY = '48410dec9118d5fcddb9ce4be1d5e387'
  API_SECRET = '0f1be97a8ec091f6'

  ACCESS_TOKEN = '72157629811083772-7ce62b48f55d76e6'
  ACCESS_SECRET = '8011fd19f6a0c19e'


  module Authorization
    class << self
      def authorize
        token, auth_url = get_auth_url
        verifier = verify(auth_url)
        log_in(token, verifier)
      end

      def get_auth_url
        FlickRaw.api_key = API_KEY
        FlickRaw.shared_secret = API_SECRET
        token = flickr.get_request_token
        auth_url = flickr.get_authorize_url(token['oauth_token'])
        return token, auth_url
      end

      def verify(auth_url)
        puts '1. Load: ' + auth_url
        puts '2. Authorize Rubber Duckie'
        puts '3. Copy/paste the authorization number here:'
        gets.strip
      end

      def log_in(token, verifier)
        oauth_token = token['oauth_token']
        oauth_token_secret = token['oauth_token_secret']
        flickr.get_access_token(oauth_token, oauth_token_secret, verifier)
        login = flickr.test.login
        puts 'Username: ' + login.username
        puts 'Access token: ' + flickr.access_token
        puts 'Access secret: ' + flickr.access_secret
      end
    end
  end


  module Search
    MPAA_RATING_TO_FLICKR_SAFE_SEARCH_VALUE = {
      rated_pg: '1',
      rated_pg13: '2',
      rated_r: '3',
    }

    MAX_RESULTS = 100

    @@logger = Logger.new(STDOUT)
    @@logger.level = Logger::INFO

    class << self
      def log_in
        FlickRaw.api_key = API_KEY
        FlickRaw.shared_secret = API_SECRET
        flickr.access_token = ACCESS_TOKEN
        flickr.access_secret = ACCESS_SECRET
        flickr.test.login
      end

      def unsafe_search(query)
        results = []
        time = Timer.time do
          threads, rated_r, rated_pg13, rated_pg = [], [], [], []
          ['rated_r', 'rated_pg13', 'rated_pg'].each do |rating|
            threads << Thread.new do
              eval("#{rating} = search(query, '#{rating}'.to_sym)")
            end
          end
          threads.each { |thread| thread.join }

          rated_r_only = rated_r - rated_pg13
          rated_pg13_only = rated_pg13 - rated_pg
          results = rated_r_only + rated_pg13_only
          results = results[0 .. MAX_RESULTS - 1]
          results = ids_to_info(results)
        end
        @@logger.info("#{query}: got #{results.size} Rated R and PG-13 photos in #{'%.2f' % time} seconds")
        results
      end

      def search(query, mpaa_rating)
        results = flickr.photos.search(
          text: query,
          sort: 'relevance',
          safe_search: MPAA_RATING_TO_FLICKR_SAFE_SEARCH_VALUE[mpaa_rating],
          per_page: '500',
        )
        results.map { |result| result['id'] }
      end

      def ids_to_info(ids)
        threads, results = [], []
        (0 .. ids.size - 1).each do |index|
          threads << Thread.new do
            info = flickr.photos.getInfo(photo_id: ids[index])
            result = {
              # title: info.title,
              # description: info.description,
              url: FlickRaw.url(info),
            }
            Thread.current[:output] = result
          end
        end
        threads.each do |thread|
          thread.join
          results << thread[:output]
        end
        results
      end
    end
  end
end


if __FILE__ == $0
  query = ARGV.join(' ')
  Flickr::Search.log_in
  puts Flickr::Search.unsafe_search(query)
end
